package com.javaonpdas.client;

import java.io.*;
import java.util.*;
import java.net.*;
import java.text.Format;
import java.text.SimpleDateFormat;

import org.apache.axis.AxisFault;
import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import javax.xml.rpc.ParameterMode;
import javax.xml.namespace.QName;
import javax.xml.rpc.encoding.XMLType;
import org.apache.axis.encoding.ser.BeanSerializerFactory;
import org.apache.axis.encoding.ser.BeanDeserializerFactory;

import com.javaonpdas.webservices.ImageValue;

public class ImageServiceClient
{
  public static void main(String[] args) throws Exception {
    // Process the command line options. We are expecting a
    // URL for the SOAP end point.
    if (args.length != 1) {
      System.err.println("Usage:");
      System.err.println("  java " +
        ImageServiceClient.class.getName() +
        " SOAP-router-URL");
      System.exit(1);
    }

    // Process the arguments.
    URL endPointURL = new URL(args[0]);

    // Set up the SOAP service
    Service service = new Service();
    Call getNamesCall = (Call)service.createCall();
    Call getImageCall = (Call)service.createCall();

    try {
      // Get the image names
      getNamesCall.setTargetEndpointAddress(endPointURL);
      getNamesCall.setOperationName(
        new QName("ImageService", "getNames"));
      getNamesCall.addParameter("extension",
        org.apache.axis.encoding.XMLType.XSD_STRING,
        ParameterMode.IN);
      getNamesCall.setReturnType(XMLType.SOAP_ARRAY);

      String[] names = (String[])getNamesCall.invoke(
        new Object[] { ".png" });
      if (names == null) {
        System.out.println("The array of names is null");
      }
      else {
        System.out.println("Image names:");
        for (int i=0; i<names.length; i++)
          System.out.println("  " + names[i]);
      }

      // Get the images
      String imageName = "kangaroo1.png";
      getImageCall.setTargetEndpointAddress(endPointURL);
      getImageCall.setOperationName(new QName("ImageService",
        "getImage"));
      getImageCall.addParameter("name",
        org.apache.axis.encoding.XMLType.XSD_STRING,
        ParameterMode.IN);
      QName qn = new QName("urn:BeanService", "ImageValue");
      getImageCall.registerTypeMapping(ImageValue.class, qn,
        new BeanSerializerFactory(ImageValue.class, qn),
        new BeanDeserializerFactory(ImageValue.class, qn));
      getImageCall.setReturnType(qn);

      ImageValue imageValue = (ImageValue)getImageCall.invoke(
        new Object[] { imageName });
      if (imageValue == null) {
        System.out.println("imageValue is null");
      }
      else {
        System.out.println("Image returned is " + imageName);
        Format formatter = new SimpleDateFormat(
          "E, dd MMM yyyy HH:mm:ss");
        String dateString = formatter.format(
          new Date(imageValue.getDate()));
        System.out.println("  image date is " + dateString);
        byte[] rawImage =
          org.apache.axis.encoding.Base64.decode(
            imageValue.getEncodedImage());
        System.out.println("  image is " + rawImage.length +
          " bytes long");
      }
    } catch (AxisFault fault) {
      System.err.println("Generated fault: ");
      System.out.println("  Fault Code   = " +
        fault.getFaultCode());
      System.out.println("  Fault String = " +
        fault.getFaultString());
    }
  }
}
