package com.javaonpdas.proxy;

import java.io.*;
import java.text.*;
import java.util.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.apache.axis.AxisFault;
import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import javax.xml.rpc.ParameterMode;
import javax.xml.namespace.QName;
import javax.xml.rpc.encoding.XMLType;
import org.apache.axis.encoding.ser.BeanSerializerFactory;
import org.apache.axis.encoding.ser.BeanDeserializerFactory;

import com.javaonpdas.webservices.ImageValue;

public class ImageServiceStreamProxy extends HttpServlet {

  private String[] getNames(URL endPointURL) {
    String[] names = null;
    try {
      Service service = new Service();
      Call call = (Call)service.createCall();
      call.setTargetEndpointAddress(endPointURL);
      call.setOperationName(new QName("ImageService",
        "getNames"));
      call.addParameter("extension",
        org.apache.axis.encoding.XMLType.XSD_STRING,
        ParameterMode.IN);
      call.setReturnType(XMLType.SOAP_ARRAY);
      names = (String[])call.invoke(new Object[]
        { ".png" });
    }
    catch (AxisFault fault) {
      System.err.println("Generated fault: ");
      System.out.println("  Fault Code   = " +
        fault.getFaultCode());
      System.out.println("  Fault String = " +
        fault.getFaultString());
    }
    catch (Exception e) {
      System.out.println(e.toString());
    }
    return names;
  }

  private ImageValue getImage(URL endPointURL, String name) {
    ImageValue imageValue = null;
    try {
      Service service = new Service();
      Call call = (Call)service.createCall();
      call.setTargetEndpointAddress(endPointURL);
      call.setOperationName(new QName("ImageService",
        "getImage"));
      call.addParameter("name",
        org.apache.axis.encoding.XMLType.XSD_STRING,
        ParameterMode.IN);
      QName qn = new QName("urn:BeanService",
        "ImageValue");
      call.registerTypeMapping(ImageValue.class, qn,
        new BeanSerializerFactory(ImageValue.class, qn),
        new BeanDeserializerFactory(ImageValue.class, qn));
      call.setReturnType(qn);

      imageValue = (ImageValue)call.invoke(new Object[] {name});
    }
    catch (AxisFault fault) {
      System.err.println("Generated fault: ");
      System.out.println("  Fault Code   = " +
        fault.getFaultCode());
      System.out.println("  Fault String = " +
        fault.getFaultString());
    }
    catch (Exception e) {
      System.out.println(e.toString());
    }
    return imageValue;
  }

  public void doGet(HttpServletRequest request,
    HttpServletResponse response) throws IOException,
    ServletException {
    URL endPointURL = new
      URL(request.getParameter("service-end-point"));
    String action = request.getParameter("action");
    if (action.equalsIgnoreCase("getNames")) {
      String[] names = getNames(endPointURL);
      DataOutputStream dos = null;
      try {
        // Create the output stream, write the number
        // of names and then the names themselves
        dos = new DataOutputStream(response.getOutputStream());
        if (names != null) {
          dos.writeInt(names.length);
          for (int i=0; i<names.length; i++)
            dos.writeUTF(names[i]);
        }
        else
          dos.writeInt(0);
      }
      catch (Exception e) {
        System.out.println(e.toString());
      }
      finally {
        try { if (dos != null) dos.close(); }
        catch (Exception e) {}
      }
    }
    else if (action.equalsIgnoreCase("getImage")) {
      String name = request.getParameter("name");
      ImageValue imageValue = getImage(endPointURL, name);
      if (imageValue == null) {
        System.out.println("imageValue is null");
      }
      else {
        byte[] byteArray =
          org.apache.axis.encoding.Base64.decode(
              imageValue.getEncodedImage());
        System.out.println("image length="+byteArray.length);
        response.setContentLength(byteArray.length);
        DataOutputStream dos = null;
        try {
          dos = new DataOutputStream(
              response.getOutputStream());
          dos.write(byteArray, 0, byteArray.length);
        }
        catch (Exception e) {
          System.out.println(e.toString());
        }
        finally {
          try { if (dos != null) dos.close(); }
          catch (Exception e) {}
        }
      }
    }
    else {
      // action not recognised
    }
  }
}
