package com.javaonpdas.persistence.threadedrms;

import javax.microedition.rms.*;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import java.io.*;
import java.util.*;

public class ThreadedRmsMIDlet extends MIDlet
  implements CommandListener, RecordComparator, RecordFilter,
  RecordListener {

  private Form mainForm = new Form("ThreadedRmsMIDlet");
  private StringItem resultItem = new StringItem("", "");
  private Command startCommand = new Command("Start",
    Command.SCREEN, 1);
  private Command saveCommand = new Command("Save",
    Command.SCREEN, 1);
  private Command getCommand = new Command("Get",
    Command.SCREEN, 1);
  private Command infoCommand = new Command("Info",
    Command.SCREEN, 1);
  private Command deleteCommand = new Command("Delete",
    Command.SCREEN, 1);
  private Command exitCommand = new Command("Exit",
    Command.EXIT, 10);
  private final String recordStoreName = "MyStore";

  private final Movie[] movies = {
    new Movie("The Patriot",
              "Mel Gibson, Heath Ledger, Joely Richardson",
              2000),
    new Movie("Gladiator",
              "Russell Crowe, Joaqin Phoenix, Connie Nielsen",
              2000),
    new Movie("Swordfish",
              "John Travolta, Hugh Jackman",
              2001),
    new Movie("Final Fantasy",
              "Alec Baldwin, Steve Buscemi",
              2001),
    new Movie("The Terminator",
              "Arnold Schwarzenegger, Michael Biehn",
              1984),
    new Movie("The Matrix",
              "Keanu Reeves, Laurence Fishburne",
              1999)};

  public ThreadedRmsMIDlet() {
    mainForm.append(resultItem);
    mainForm.addCommand(startCommand);
    mainForm.addCommand(saveCommand);
    mainForm.addCommand(getCommand);
    mainForm.addCommand(infoCommand);
    mainForm.addCommand(deleteCommand);
    mainForm.addCommand(exitCommand);
    mainForm.setCommandListener(this);
  }

  public void startApp() {
    Display.getDisplay (this).setCurrent (mainForm);
  }

  public void pauseApp() {
  }

  public void destroyApp(boolean unconditional) {
  }

  public void commandAction(Command c, Displayable d) {
    RecordStore store = null;
    try {
      if (c == startCommand) {
        store = RecordStore.openRecordStore(recordStoreName,
          true);
        store.addRecordListener(this);
        RmsThread thread = new RmsThread(5, movies, store);
        thread.start();
      }
      if (c == saveCommand) {
        store = RecordStore.openRecordStore(recordStoreName,
          true);
        for (int i=0; i<movies.length; i++) {
          ByteArrayOutputStream bos =
            new ByteArrayOutputStream();
          DataOutputStream dos = new DataOutputStream(bos);
          // write the object to the stream
          movies[i].writeObject(dos);
          byte[] ba = bos.toByteArray();
          store.addRecord(ba, 0, ba.length);
        }
        store.closeRecordStore();
        resultItem.setLabel ("Status:");
        resultItem.setText(movies.length + " records written");
      }
      else if (c == getCommand) {
        store = RecordStore.openRecordStore(recordStoreName,
          false);
        StringBuffer result = new StringBuffer();
        RecordEnumeration re = store.enumerateRecords(this,
          this, false);
        // RecordEnumeration re = store.enumerateRecords(null,
        //   this, false);
        int i=1;
        while(re.hasNextElement()) {
          ByteArrayInputStream bis = new ByteArrayInputStream(
            re.nextRecord());
          DataInputStream dis = new DataInputStream(bis);
          Movie movie = new Movie();
          movie.readObject(dis);
          result.append(i++ + ": " + movie.title + '\n');
        }
        store.closeRecordStore();
        resultItem.setLabel ("Status:");
        resultItem.setText(result.toString());
      }
      else if (c == infoCommand) {
        store = RecordStore.openRecordStore(recordStoreName,
          false);
        StringBuffer result = new StringBuffer();
        result.append("Name: " + store.getName() + "\n");
        result.append("Records: " + store.getNumRecords() +
          '\n');
        result.append("Store size: " + store.getSize() +
          " bytes\n");
        result.append("Bytes available: " +
          store.getSizeAvailable() + " bytes\n");
        result.append("Version: " + store.getVersion() + "\n");
        resultItem.setLabel("Info:");
        resultItem.setText(result.toString());
      }
      else if (c == deleteCommand) {
        RecordStore.deleteRecordStore(recordStoreName);
        resultItem.setLabel ("Status:");
        resultItem.setText(recordStoreName + " deleted.");
      }
      else if (c == exitCommand) {
        destroyApp(false);
        notifyDestroyed();
      }
    }
    catch (Exception e) {
      e.printStackTrace ();
      resultItem.setLabel ("Error:");
      resultItem.setText (e.toString ());
    }
  }

  public int compare(byte[] rec1, byte[] rec2)
  {
    Movie movie1 = null;
    Movie movie2 = null;
    try {
      ByteArrayInputStream bis1 =
        new ByteArrayInputStream(rec1);
      DataInputStream dis1 = new DataInputStream(bis1);
      movie1 = new Movie();
      movie1.readObject(dis1);
      ByteArrayInputStream bis2 =
        new ByteArrayInputStream(rec2);
      DataInputStream dis2 = new DataInputStream(bis2);
      movie2 = new Movie();
      movie2.readObject(dis2);
    }
    catch (Exception e) {
      System.out.println(e);
      e.printStackTrace();
    }

    // sort by title
    int result = movie1.title.compareTo(movie2.title);
    if (result < 0) {
      return RecordComparator.PRECEDES;
    }
    else if (result > 0) {
      return RecordComparator.FOLLOWS;
    }
    else {
      return RecordComparator.EQUIVALENT;
    }
  }

  public boolean matches(byte[] candidate) {
    boolean result = true;
    Movie movie = null;
    try {
      ByteArrayInputStream bis =
        new ByteArrayInputStream(candidate);
      DataInputStream dis = new DataInputStream(bis);
      movie = new Movie();
      movie.readObject(dis);
    }
    catch (Exception e) {
      System.out.println(e);
      e.printStackTrace();
    }
    result = movie.title.startsWith("The");
    return result;
  }

  Movie getMovieFromRecord(RecordStore store, int recordId) {
    Movie movie = new Movie();
    try {
      ByteArrayInputStream bis =
        new ByteArrayInputStream(store.getRecord(recordId));
      DataInputStream dis = new DataInputStream(bis);
      movie.readObject(dis);
    }
    catch (Exception e) {
      System.out.println(e);
      e.printStackTrace();
    }
    return movie;
  }

  public void recordAdded(RecordStore store, int recordId) {
    resultItem.setLabel ("Status:");
    resultItem.setText("Movie added: " +
      getMovieFromRecord(store, recordId).title);
  }

  public void recordChanged(RecordStore store, int recordId) {
    resultItem.setLabel ("Status:");
    resultItem.setText("Movie updated: " +
      getMovieFromRecord(store, recordId).title);
  }

  public void recordDeleted(RecordStore store, int recordId) {
    resultItem.setLabel ("Status:");
    resultItem.setText("Movie deleted: " + recordId);
  }
}
